/*
    DoubleSimple: DoubleRoundCmpEpsilon.m
    Copyright (C) 2003-2006  Macaulay Institute

    This file is part of DoubleSimple, a series of classes for implementing
    various remedies to floating point arithmetic errors.

    DoubleSimple is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DoubleSimple is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

Implementation of DoubleRoundCmpEpsilon.

*/

#import <DoubleRoundCmpEpsilon.h>
#import "purify.h"

#define OPERATOR(op, arg) \
  int sigfig = [self getSignificantFigures]; \
  double epsilon = [self getEpsilon]; \
  return purify_round_epsilon(d, sigfig, epsilon) \
    op purify_round_epsilon(arg, sigfig, epsilon) ? YES : NO

@implementation DoubleRoundCmpEpsilon

-(BOOL)eq: (DoubleSimple *)value {
  OPERATOR(==, [value get]);
}

-(BOOL)ne: (DoubleSimple *)value {
  OPERATOR(!=, [value get]);
}

-(BOOL)gt: (DoubleSimple *)value {
  OPERATOR(>, [value get]);
}

-(BOOL)le: (DoubleSimple *)value {
  OPERATOR(<=, [value get]);
}

-(BOOL)ge: (DoubleSimple *)value {
  OPERATOR(>=, [value get]);
}

-(BOOL)lt: (DoubleSimple *)value {
  OPERATOR(<, [value get]);
}


-(BOOL)ieq: (int)value {
  OPERATOR(==, (double)value);
}

-(BOOL)ine: (int)value {
  OPERATOR(!=, (double)value);
}

-(BOOL)igt: (int)value {
  OPERATOR(>, (double)value);
}

-(BOOL)ile: (int)value {
  OPERATOR(<=, (double)value);
}

-(BOOL)ige: (int)value {
  OPERATOR(>=, (double)value);
}

-(BOOL)ilt: (int)value {
  OPERATOR(<, (double)value);
}


-(BOOL)ceq: (double)value {
  OPERATOR(==, value);
}

-(BOOL)cne: (double)value {
  OPERATOR(!=, value);
}

-(BOOL)cgt: (double)value {
  OPERATOR(>, value);
}

-(BOOL)cle: (double)value {
  OPERATOR(<=, value);
}

-(BOOL)cge: (double)value {
  OPERATOR(>=, value);
}

-(BOOL)clt: (double)value {
  OPERATOR(<, value);
}

@end
