/*
    DoubleSimple: DoubleSigFig.m
    Copyright (C) 2003-2006  Macaulay Institute

    This file is part of DoubleSimple, a series of classes for implementing
    various remedies to floating point arithmetic errors.

    DoubleSimple is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DoubleSimple is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

Implementation of DoubleSigFig class

*/

#import <DoubleSigFig.h>
#import <float.h>

/* Class variable */

static int significant_figures = DBL_DIG;

@implementation DoubleSigFig


/* New methods for this class */

+setSignificantFigures: (int)sf {
  if(sf < 1) {
    fprintf(stderr, "Invalid setting for significant figures: %d\n", sf);
    abort();
  }
  if(sf > DBL_DIG) {
    fprintf(stderr, "WARNING: DoubleString used with more significant figures"
	    " than guaranteed by this platform for double precision floating"
	    " point: %d (%d)\n", DBL_DIG, sf);
  }
  significant_figures = sf;
  return self;
}

-setSignificantFigures: (int)sf {
  [[self class] setSignificantFigures: sf];

  return self;
}

+(int)getSignificantFigures {
  return significant_figures;
}

-(int)getSignificantFigures {
  return [[self class] getSignificantFigures];
}

/* Methods to override from DoubleSimple */

+parseValue: (char *)str {
  char *p = strchr(str, (int)'s');

  if(p != NULL) {
    (*p) = '\0';
    p++;
    [self setSignificantFigures: atoi(p)];
  }
  return [super parseValue: str];
}

@end
